@@ -3,7 +3,8 @@ import numpy as np
 from guide_bot.base_elements import guide_elements
 from guide_bot.parameters import instrument_parameters as ipars
 
-from guide_bot.base_elements.guide_elements import PositionAndRotation
+from guide_bot.base_elements.base_element_geometry import BaseElementGeometry
+from guide_bot.base_elements.base_element_geometry import PositionAndRotation
 
 class Elliptic(guide_elements.GuideElement):
     """
@@ -88,7 +89,6 @@ class Elliptic(guide_elements.GuideElement):
                          end_width=end_width, end_height=end_height,
                          **kwargs)
 
-
         self.max_width = max_width
         if self.max_width is None:
             max_width = 0.3
@@ -160,11 +160,6 @@ class Elliptic(guide_elements.GuideElement):
         else:
             self.enableGravity = 1
 
-    def write_to_log(self, file):
-        self.write_to_log_base(file)
-        self.write_parameter(file, "minor_axis_x", self.minor_axis_x, "horizontal")
-        self.write_parameter(file, "minor_axis_y", self.minor_axis_y, "vertical")
-
     def add_to_instr(self):
         """
         Adds code describing the elliptic element to the current McStas instrument
@@ -258,30 +253,50 @@ class Elliptic(guide_elements.GuideElement):
         instr = self.current_instrument
 
         # Calculate ellipse parameters in McStas instrument
-        instr.add_declare_var("double", "tmp_w1")
-        instr.add_declare_var("double", "tmp_w2")
+        try:
+            # If multiple elliptic guides, multiple declare of the same variable becomes an error
+            instr.add_declare_var("double", "tmp_w1")
+            instr.add_declare_var("double", "tmp_w2")
+        except:
+            pass
         instr.append_initialize('')
         instr.append_initialize('tmp_w1=' + self.start_width.name + ';')
         instr.append_initialize('tmp_w2=' + self.end_width.name + ';')
 
-        instr.add_declare_var("double", "tmp_b")
-        instr.add_declare_var("double", "tmp_L")
+        try:
+            # If multiple elliptic guides, multiple declare of the same variable becomes an error
+            instr.add_declare_var("double", "tmp_b")
+            instr.add_declare_var("double", "tmp_L")
+        except:
+            pass
         instr.append_initialize('tmp_b=fabs(0.5*' + self.minor_axis_x.name + ');')
         instr.append_initialize('tmp_L=' + length_name + ';')
 
-        instr.add_declare_var("double", "tmp_k")
+        try:
+            # If multiple elliptic guides, multiple declare of the same variable becomes an error
+            instr.add_declare_var("double", "tmp_k")
+        except:
+            pass
         instr.append_initialize('if (' + self.minor_axis_x.name + ' > 0) {')
         instr.append_initialize('tmp_k=cos(asin(tmp_w2/(2*tmp_b)))/cos(asin(tmp_w1/(2*tmp_b)));')
         instr.append_initialize('} else {')
         instr.append_initialize('tmp_k=cos(asin(tmp_w2/(2*tmp_b)))/cos(PI-asin(tmp_w1/(2*tmp_b)));')
         instr.append_initialize('}')
 
-        instr.add_declare_var("double", "tmp_L1")
-        instr.add_declare_var("double", "tmp_L2")
+        try:
+            # If multiple elliptic guides, multiple declare of the same variable becomes an error
+            instr.add_declare_var("double", "tmp_L1")
+            instr.add_declare_var("double", "tmp_L2")
+        except:
+            pass
         instr.append_initialize('tmp_L1=tmp_L/(1+tmp_k);')
         instr.append_initialize('tmp_L2=tmp_L-tmp_L1;')
 
-        instr.add_declare_var("double", "tmp_c")
+        try:
+            # If multiple elliptic guides, multiple declare of the same variable becomes an error
+            instr.add_declare_var("double", "tmp_c")
+        except:
+            pass
         instr.append_initialize('if (' + self.minor_axis_x.name + ' > 0) {')
         instr.append_initialize('tmp_c=cos(asin(tmp_w1/(2*tmp_b)))*cos(asin(tmp_w1/(2*tmp_b)));')
         instr.append_initialize('} else {')
@@ -296,8 +311,12 @@ class Elliptic(guide_elements.GuideElement):
         instr.append_initialize(Loutx_par_name + '=' + Linx_par_name + '+tmp_L1-tmp_L2;')
 
         instr.append_initialize('')
-        instr.add_declare_var("double", "tmp_h1")
-        instr.add_declare_var("double", "tmp_h2")
+        try:
+            # If multiple elliptic guides, multiple declare of the same variable becomes an error
+            instr.add_declare_var("double", "tmp_h1")
+            instr.add_declare_var("double", "tmp_h2")
+        except:
+            pass
         instr.append_initialize('tmp_h1=' + self.start_height.name + ';')
         instr.append_initialize('tmp_h2=' + self.end_height.name + ';')
 
@@ -352,7 +371,44 @@ class Elliptic(guide_elements.GuideElement):
         end = self.current_instrument.add_component(self.end_component_name, "Arm")
         end.set_AT([0, 0, self.get_length_name()], RELATIVE=guide)
 
-import numpy as np
+    def get_geometry(self):
+        return GeometryElliptic(start_point=self.start_point_parameter,
+                                next_start_point=self.next_start_point_parameter,
+                                start_width=self.start_width, start_height=self.start_height,
+                                end_width=self.end_width, end_height=self.end_height,
+                                minor_axis_x=self.minor_axis_x, minor_axis_y=self.minor_axis_y)
+
+
+class GeometryElliptic(BaseElementGeometry):
+    def __init__(self, start_point, next_start_point, start_width, start_height, end_width, end_height,
+                 minor_axis_x, minor_axis_y):
+
+        super().__init__(start_point=start_point, next_start_point=next_start_point,
+                         start_width=start_width, start_height=start_height,
+                         end_width=end_width, end_height=end_height)
+
+        self.set_parameter_attribute("minor_axis_x", minor_axis_x, par_type="horizontal")
+        self.set_parameter_attribute("minor_axis_y", minor_axis_y, par_type="vertical")
+
+        self.plot_resolution = 60  # Number of points to plot
+
+    def get_dimension(self, distance_unitless, horizontal):
+        start = self.start_point
+        end = self.next_start_point
+
+        if horizontal:
+            start_dim = self.start_width
+            end_dim = self.end_width
+            minor_axis = self.minor_axis_x
+        else:
+            start_dim = self.start_height
+            end_dim = self.end_height
+            minor_axis = self.minor_axis_y
+
+        ellipse_pos = (end - start) * distance_unitless
+        ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_dim, end_dim,
+                                                             minor_axis)
+        return 2.0 * ellipse_p_side
 
 
 def ellipse_calculation(ellipse_pos, start, end, start_dim, end_dim, minor_axis):
@@ -405,87 +461,3 @@ def ellipse_calculation(ellipse_pos, start, end, start_dim, end_dim, minor_axis)
 
     return ellipse_p_side, ellipse_n_side
 
-def plot_element_horizontal(ax, par_dict, color):
-
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_width = par_dict["start_width"]
-    end_width = par_dict["end_width"]
-    minor_axis = par_dict["minor_axis_x"]
-
-    ellipse_pos = np.linspace(0, end - start, 100)
-    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_width, end_width, minor_axis)
-
-    ax.plot(start + ellipse_pos, ellipse_p_side, color=color)
-    ax.plot(start + ellipse_pos, ellipse_n_side, color=color)
-
-
-def plot_element_vertical(ax, par_dict, color):
-
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_height = par_dict["start_height"]
-    end_height = par_dict["end_height"]
-    minor_axis = par_dict["minor_axis_y"]
-
-    ellipse_pos = np.linspace(0, end - start, 100)
-    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_height, end_height, minor_axis)
-
-    ax.plot(start + ellipse_pos, ellipse_p_side, color=color)
-    ax.plot(start + ellipse_pos, ellipse_n_side, color=color)
-
-def guide_width(par_dict, distance_unitless):
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_width = par_dict["start_width"]
-    end_width = par_dict["end_width"]
-    minor_axis = par_dict["minor_axis_x"]
-
-    ellipse_pos = (end-start)*distance_unitless
-    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_width, end_width, minor_axis)
-    return 2.0 * ellipse_p_side
-
-def guide_height(par_dict, distance_unitless):
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_height = par_dict["start_height"]
-    end_height = par_dict["end_height"]
-    minor_axis = par_dict["minor_axis_y"]
-
-    ellipse_pos = (end-start)*distance_unitless
-    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_height, end_height, minor_axis)
-    return 2.0 * ellipse_p_side
-
-def guide_dimensions(par_dict, distance_unitless, horizontal):
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-
-    if horizontal:
-        start_dim = par_dict["start_width"]
-        end_dim = par_dict["end_width"]
-        minor_axis = par_dict["minor_axis_x"]
-    else:
-        start_dim = par_dict["start_height"]
-        end_dim = par_dict["end_height"]
-        minor_axis = par_dict["minor_axis_y"]
-
-    ellipse_pos = (end - start) * distance_unitless
-    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_dim, end_dim, minor_axis)
-    return 2.0 * ellipse_p_side
-
-
-def center_line(start_pr, par_dict, distance_unit_less):
-
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-
-    length = end - start
-    this_position = length*distance_unit_less
-
-    z_vector = np.array([0, 0, 1])
-    start_direction = start_pr.rotation
-    z_vector = start_direction.apply(z_vector)
-
-    updated_position = start_pr.position + this_position*z_vector
-
-    return PositionAndRotation(updated_position, start_pr.rotation)
\ No newline at end of file
