@@ -8,6 +8,8 @@ from guide_bot.parameters.instrument_parameters import FreeInstrumentParameter
 from guide_bot.parameters.instrument_parameters import RelativeFreeInstrumentParameter
 from guide_bot.parameters.instrument_parameters import FixedInstrumentParameter
 from guide_bot.parameters.instrument_parameters import DependentInstrumentParameter
+from guide_bot.logic.line_of_sight import LosCalculator
+from guide_bot.logic.line_of_sight import LosError
 
 from guide_bot.logging import guide_log
 
@@ -49,10 +51,17 @@ def optimizer_func(optimizer_x, *args, **kwargs):
 
     instrument = kwargs["instrument"]
     instr_parameters = kwargs["instrument_parameters"]
+    guide = kwargs["guide"]
+    los_calculator = kwargs["los_calculator"]
     settings = kwargs["settings"]
     logfile = kwargs["logfile"]
 
-    instr_parameter_input = instr_parameters.extract_instrument_parameters(optimizer_x)
+    try:
+        instr_parameter_input = instr_parameters.extract_instrument_parameters(optimizer_x, los_calculator=los_calculator)
+    except LosError:
+        # If los can not be solved, return 0.0
+        # Can't write in the parameter file as no instrument parameters returned after LosError
+        return 0.0
 
     if not settings["logfile"]:
         print(instr_parameters)
@@ -69,6 +78,11 @@ def optimizer_func(optimizer_x, *args, **kwargs):
     if not settings["logfile"]:
         print("Running with \n", instr_parameter_input)
 
+    # debug print
+    #print(instr_parameter_input)
+    #guide.print_start_points()
+    #guide.plot_guide()
+
     sim_data = instrument.run_full_instrument(foldername=settings["foldername"],
                                               increment_folder_name=True, force_compile=False,
                                               parameters=instr_parameter_input, ncount=settings["ncount"])
@@ -123,7 +137,7 @@ def optimizer_f_ieqcons(optimizer_x, *args, **kwargs):
     return instr_parameters.evaluate_constraints(optimizer_x)
 
 
-def run_optimizer(instrument, my_parameters, settings, scan_name):
+def run_optimizer(instrument, my_parameters, guide, settings, scan_name):
     """
     Performs optimization of given instrument with parameters and settings
 
@@ -145,8 +159,14 @@ def run_optimizer(instrument, my_parameters, settings, scan_name):
     my_parameters : InstrumentParameterContainer
         Parameters and constraints for the optimization
 
+    guide : Guide
+        Description of the guide under optimization
+
     settings : dict
         Dict with options for the optimization
+
+    scan_name : str
+        String describing this scan_name (used in filenames)
     """
     instrument.write_full_instrument()
 
@@ -165,19 +185,22 @@ def run_optimizer(instrument, my_parameters, settings, scan_name):
         print("No free parameters, optimization skipped.")
         return []
 
+    los_calculator = LosCalculator(guide)
+
     lb = my_parameters.get_lower_bounds()
     ub = my_parameters.get_upper_bounds()
 
     my_history = guide_log.History()
 
-    kw_package = {"instrument": instrument, "instrument_parameters": my_parameters,
-                  "settings": settings, "history": my_history, "logfile": logfile}
+    kw_package = {"instrument": instrument, "instrument_parameters": my_parameters, "guide": guide,
+                  "settings": settings, "history": my_history, "logfile": logfile, "los_calculator": los_calculator}
 
     xopt, fopt = pyswarm.pso(optimizer_func, lb, ub, f_ieqcons=optimizer_f_ieqcons, kwargs=kw_package,
                              swarmsize=settings["swarmsize"], omega=settings["omega"], phip=settings["phip"],
                              phig=settings["phig"], maxiter=settings["maxiter"], minstep=settings["minstep"],
                              minfunc=settings["minfunc"], debug=False)
 
+    """
     print("-"*30, "done", "-"*50)
     print("best x:", xopt)
     print("best fom: ", fopt)
@@ -186,7 +209,9 @@ def run_optimizer(instrument, my_parameters, settings, scan_name):
     lowest_fom = 1
     lowest_fom_pars = None
     for history in my_history.histories:
-        instrument_pars = my_parameters.extract_instrument_parameters(history.optimized_x)
+        my_parameters.set_values(history.optimized_x, los_calculator=los_calculator)
+        instrument_pars = my_parameters.extract_instrument_parameters()
+
         print(history.fom, "\tFrom this set:", instrument_pars)
         if history.fom < lowest_fom:
             lowest_fom = history.fom
@@ -207,10 +232,12 @@ def run_optimizer(instrument, my_parameters, settings, scan_name):
     print("redone fom", fom)
     print("fom in optimization: ", lowest_fom)
     print("ratio = ", fom/lowest_fom)
+    
 
     print("For par file:")
     for key in lowest_fom_pars:
         print(key + "=" + str(lowest_fom_pars[key]))
+    """
 
     return xopt
 
